@@ -32,66 +32,32 @@
 ;;; Read the docstrings; read the texinfo file.
 
 ;;
-;; $Id: bbdb-hooks.el,v 1.63 2001/01/29 14:43:07 waider Exp $
-;;
-;; $Log: bbdb-hooks.el,v $
-;; Revision 1.63  2001/01/29 14:43:07  waider
-;; Added autoload cookie for bbdb-ignore-selected-messages-hook
-;;
-;; Revision 1.62  2001/01/08 12:35:14  waider
-;; Added Bill Carpenter-provided function
-;;   'bbdb-ignore-selected-messages-confirmation'
-;;
-;; Revision 1.61  2000/11/16 11:59:40  fenk
-;; (bbdb-extract-field-value): added
-;;  (case-fold-search t) as headers should be checked case insensitive
-;;
-;; Revision 1.60  2000/08/03 18:04:50  sds
-;; * lisp/bbdb.el (bbdb-notes-default-separator): new user option
-;; (bbdb-annotate-notes): use it
-;; (notes, company): put `field-separator' property
-;; * lisp/bbdb-hooks.el (bbdb-auto-notes-hook): search the whole
-;; notes string for the new note before adding
-;;
-;; Revision 1.59  2000/07/13 17:07:00  sds
-;; minor doc fixes to comply with the standards
-;;
-;; Revision 1.58  2000/04/05 17:09:06  bbdb-writer
-;; * Autoload cookie for bbdb-header-start
-;;
-;; Revision 1.57  1998/04/11 07:20:59  simmonmt
-;; Colin Rafferty's patch adding autoload cookies back.
-;; Made `format-time-string' take two arguments for XEmacs 19.15.
-;;
-;; Revision 1.56  1998/03/10 07:37:13  simmonmt
-;; Fixed customization of bbdb-auto-notes-alist
-;;
-;; Revision 1.55  1998/02/23 07:09:37  simmonmt
-;; We use add-hook now
-;;
-;; Revision 1.54  1998/01/06 06:05:31  simmonmt
-;; Added provide of bbdb-hooks.  Fixed custom specs (added cons type
-;; instead of group where appropriate).  Replaced bbdb-time-string
-;; function with bbdb-time-internal-format variable.
-;;
-;; Revision 1.53  1997/12/01 05:00:49  simmonmt
-;; Customized, added sshteingold@cctrading.com's change to time-string
-;; function to use a format string.
-;;
-;; Revision 1.52  1997/09/28 06:01:05  simmonmt
-;; Fix to accomodate nil gnus-single-article-buffer
-;;
+;; $Id: bbdb-hooks.el,v 1.7 2007/02/23 20:24:07 fenk Exp $
 ;;
 
 (require 'bbdb)
+(require 'bbdb-com)
+(require 'mail-parse)
+
+(eval-when-compile
+  (condition-case()
+      (progn
+        (require 'gnus)
+        (require 'bbdb-gnus))
+    (error nil))
+  (condition-case()
+      (progn
+        (require 'vm)
+        (require 'bbdb-vm))
+    (error nil))
+  (autoload 'mh-show "mh-e")
+  (condition-case()
+      (require 'bbdb-rmail)
+    (error nil)))
+
+(defvar rmail-buffer)
+(defvar mh-show-buffer)
 
-(defmacro the-v18-byte-compiler-sucks-wet-farts-from-dead-pigeons ()
-  ;; no such thing as eval-when, no way to conditionally require something
-  ;; at compile time (except this!! <evil laughter> )
-  (condition-case () (require 'vm) (error nil))
-  nil)
-(defun Nukem-til-they-glow ()
-  (the-v18-byte-compiler-sucks-wet-farts-from-dead-pigeons))
 
 (defvar bbdb-time-internal-format "%Y-%m-%d"
   "The internal date format.")
@@ -118,59 +84,72 @@ which is the current time string."
 ;;; Determining whether to create a record based on the content of the
 ;;; current message.
 
+(eval-when-compile
+  (defvar vm-mail-buffer)
+  (defvar vm-message-pointer)
+  (autoload 'vm-start-of "vm")
+  (autoload 'bbdb/vm-pop-up-bbdb-buffer "bbdb-vm"))
+
 ;;;###autoload
 (defun bbdb-header-start ()
   "Returns a marker at the beginning of the header block of the current
 message.  This will not necessarily be in the current buffer."
-  (cond ((memq major-mode '(vm-mode vm-summary-mode))
-     (if vm-mail-buffer (set-buffer vm-mail-buffer))
-     (vm-start-of (car vm-message-pointer)))
-    ((memq major-mode '(rmail-mode rmail-summary-mode))
-     (if (and (boundp 'rmail-buffer) rmail-buffer)
-         (set-buffer rmail-buffer))
-     (point-min-marker))
-    ((memq major-mode
-           '(gnus-Group-mode gnus-Subject-mode gnus-Article-mode))
-     (set-buffer gnus-article-buffer)
-     (point-min-marker))
-    ;; MH-E clause added by knabe.
-    ((eq major-mode 'mh-folder-mode)
-     (mh-show)
-     (set-buffer mh-show-buffer)
-     (point-min-marker))
-    (t (point-min-marker))
-    ))
+  (cond ((memq major-mode
+	       '(gnus-group-mode gnus-summary-mode gnus-article-mode))
+	 (set-buffer (or gnus-original-article-buffer
+                         gnus-article-buffer))
+	 (point-min-marker))
+        ((memq major-mode '(vm-presentation-mode vm-mode vm-summary-mode))
+         (if vm-mail-buffer (set-buffer vm-mail-buffer))
+         (vm-start-of (car vm-message-pointer)))
+        ((memq major-mode '(rmail-mode rmail-summary-mode))
+         (if (and (boundp 'rmail-buffer) rmail-buffer)
+             (set-buffer rmail-buffer))
+         (point-min-marker))
+        ;; MH-E clause added by knabe.
+        ((eq major-mode 'mh-folder-mode)
+         (mh-show)
+         (set-buffer mh-show-buffer)
+         (point-min-marker))
+        (t (point-min-marker))
+        ))
 
 
 ;;;###autoload
 (defun bbdb-extract-field-value (field-name)
   "Given the name of a field (like \"Subject\") this returns the value of
 that field in the current message, or nil.  This works whether you're in
-GNUS, Rmail, or VM.  This works on multi-line fields, but if more than
+Gnus, Rmail, or VM.  This works on multi-line fields, but if more than
 one field of the same name is present, only the last is returned.  It is
 expected that the current buffer has a message in it, and (point) is at the
 beginning of the message headers."
   ;; we can't special-case VM here to use its cache, because the cache has
   ;; divided real-names from addresses; the actual From: and Subject: fields
   ;; exist only in the message.
-  (setq field-name (concat (regexp-quote field-name) "[ \t]*:[ \t]*"))
-  (let ((case-fold-search t)
-    done)
-    (while (not (or done
-            (looking-at "\n") ; we're at BOL
-            (eobp)))
-      (if (looking-at field-name)
-      (progn
-        (goto-char (match-end 0))
-        (setq done (buffer-substring (point)
-                     (progn (end-of-line) (point))))
-        (while (looking-at "\n[ \t]")
-          (setq done (concat done " "
-               (buffer-substring (match-end 0)
-                 (progn (end-of-line 2) (point))))))))
-      (forward-line 1))
-    done))
-
+  (save-excursion
+    (if (memq major-mode
+              '(gnus-summary-mode gnus-article-mode gnus-tree-mode))
+        (progn
+          (set-buffer (get-buffer gnus-original-article-buffer))
+          (goto-char (point-min))))
+    (setq field-name (concat (regexp-quote field-name) "[ \t]*:[ \t]*"))
+    (let ((case-fold-search t)
+          done)
+      (while (not (or done
+              (looking-at "\n") ; we're at BOL
+              (eobp)))
+        (if (looking-at field-name)
+        (progn
+          (goto-char (match-end 0))
+          (setq done (buffer-substring (point)
+                       (progn (end-of-line) (point))))
+          (while (looking-at "\n[ \t]")
+            (setq done (concat done " "
+                 (buffer-substring (match-end 0)
+                   (progn (end-of-line 2) (point))))))))
+        (forward-line 1))
+      (and done
+	   (mail-decode-encoded-word-string done)))))
 
 (defcustom bbdb-ignore-most-messages-alist '()
   "*An alist describing which messages to automatically create BBDB
@@ -221,32 +200,31 @@ the bbdb-ignore-most-messages-alist (which see) and *no* others."
   (let ((rest (if invert-sense
           bbdb-ignore-some-messages-alist
           bbdb-ignore-most-messages-alist))
-    (case-fold-search t)
-    (done nil)
-    (b (current-buffer))
-    (marker (bbdb-header-start))
-    field regexp fieldval)
+        (case-fold-search t)
+        (done nil)
+        (b (current-buffer))
+        (marker (bbdb-header-start))
+        field regexp fieldval)
     (set-buffer (marker-buffer marker))
     (save-restriction
       (widen)
       (while (and rest (not done))
-    (goto-char marker)
-    (setq field (car (car rest))
-          regexp (cdr (car rest))
-          fieldval (bbdb-extract-field-value field))
-    (if (and fieldval (string-match regexp fieldval))
-        (setq done t))
-    (setq rest (cdr rest))))
+        (goto-char marker)
+        (setq field (car (car rest))
+              regexp (cdr (car rest))
+              fieldval (bbdb-extract-field-value field))
+        (if (and fieldval (string-match regexp fieldval))
+            (setq done t))
+        (setq rest (cdr rest))))
     (set-buffer b)
     (if invert-sense
-    (not done)
-    done)))
+        (not done)
+      done)))
 
 ;;; Provided by Bill Carpenter.
-(defvar bbdb-ignore-selected-messages-confirmation
-  "*If bbdb-ignore-selected-messages-hook is used as an
-auto-create-hook, this variable governs whether you are prompted for
-creation of BBDB entries." nil)
+(defvar bbdb-ignore-selected-messages-confirmation nil
+  "*If bbdb-ignore-selected-messages-hook is used as an auto-create-hook, this
+variable governs whether you are prompted for creation of BBDB entries.")
 
 ;;;###autoload
 (defun bbdb-ignore-selected-messages-hook ()
@@ -257,7 +235,7 @@ bbdb-ignore-most-messages-alist.  It first looks at the SOME list.  If
 that doesn't disqualify a message, then it looks at the MOST list.  If
 that qualifies the message, the record is auto-created, but a
 confirmation is conditionally sought, based on the value of
-bbdb-ignore-selected-messages-confirmation."
+`bbdb-ignore-selected-messages-confirmation'."
   (if (bbdb-ignore-some-messages-hook)
       ;; wasn't ruled out
       (if (bbdb-ignore-most-messages-hook)
@@ -296,7 +274,7 @@ to the BBDB record representing the sender of the current message based on
 the subject or other header fields.  This only works if `bbdb-notice-hook'
 contains `bbdb-auto-notes-hook'.  The format of this alist is
 
-   (( HEADER-NAME
+   ((HEADER-NAME [ADDRESS-CLASS-LIST]
        (REGEXP . STRING) ... )
       ... )
 for example,
@@ -309,6 +287,14 @@ the record corresponding to anyone you get mail from via one of the VM
 mailing lists.  If, that is, `bbdb/mail-auto-create-p' is set such that the
 record would have been created, or the record already existed.
 
+A ADDRESS-CLASS-LIST is optional and by default actions will be performed only
+for records of authors of a message.  However, by giving an list of classes
+specified in `bbdb-get-addresses-headers'.  Actions will then only be
+performed if the currently processed email is of a class listed in
+ADDRESS-CLASS-LIST.  ADDRESS-CLASS-LIST might also be an alist with elements
+of the form (CLASS . HEADER) which allows actions only when the current
+address matches one of the elemets.
+
 The format of elements of this list may also be
        (REGEXP FIELD-NAME STRING)
 or
@@ -352,12 +338,30 @@ a different value when in mail as when in news.
 
 See also variables `bbdb-auto-notes-ignore' and `bbdb-auto-notes-ignore-all'."
   :group 'bbdb-noticing-records
-  :type '(repeat (bbdb-alist-with-header
-          (string :tag "Header name")
-          (repeat (cons
-               (regexp :tag "Regexp to match on header value")
-               (string :tag "String for notes if regexp matches")))
-          )))
+  :type '(repeat
+          (bbdb-alist-with-header
+           (string :tag "Header name")
+           (repeat (choice
+                    (cons :tag "Address Class"
+                          (repeat (choice
+                                   (const authors)
+                                   (const recipients))))
+                    (cons :tag "Value Pair"
+                          (regexp :tag "Regexp to match on header value")
+                          (string :tag "String for notes if regexp matches"))
+                    (list :tag "Replacement list"
+                          (regexp :tag "Regexp to match on header value")
+                          (choice :tag "Record field"
+                                  (const notes :tag "Notes")
+                                  (const company :tag "Company")
+                                  (symbol :tag "Other"))
+                          (choice :tag "Regexp match"
+                                  (string :tag "Replacement string")
+                                  (integer :tag "Subexpression match")
+                                  (function :tag "Callback Function"))
+                          (choice :tag "Replace previous contents"
+                                  (const :tag "No" nil)
+                                  (const :tag "Yes" t))))))))
 
 (defcustom bbdb-auto-notes-ignore nil
   "Alist of headers and regexps to ignore in `bbdb-auto-notes-hook'.
@@ -408,14 +412,14 @@ the variables `bbdb-auto-notes-alist' and `bbdb-auto-notes-ignore'."
   ;; cached for any other message, but that's probably not the right thing.
   (unless bbdb-readonly-p
    (let ((rest bbdb-auto-notes-alist)
-         ignore
          (ignore-all bbdb-auto-notes-ignore-all)
          (case-fold-search t)
          (b (current-buffer))
          (marker (bbdb-header-start))
+         ignore
          field pairs fieldval  ; do all bindings here for speed
          regexp string notes-field-name notes
-         replace-p replace-or-add-msg)
+         replace-p)
     (set-buffer (marker-buffer marker))
     (save-restriction
       (widen)
@@ -438,7 +442,7 @@ the variables `bbdb-auto-notes-alist' and `bbdb-auto-notes-ignore'."
             (setq ignore-all (cdr ignore-all))))
 
         (unless ignore          ; ignore-all matched
-         (while rest ; while their still are clauses in the auto-notes alist
+         (while rest ; while there are still clauses in the auto-notes alist
           (goto-char marker)
           (setq field (car (car rest))  ; name of header, e.g., "Subject"
                 pairs (cdr (car rest))  ; (REGEXP . STRING) or
@@ -446,6 +450,21 @@ the variables `bbdb-auto-notes-alist' and `bbdb-auto-notes-ignore'."
                                         ; (REGEXP FIELD-NAME STRING REPLACE-P)
                 fieldval (bbdb-extract-field-value field)) ; e.g., Subject line
           (when fieldval
+            ;; we perform the auto notes stuff only for authors of a message
+            ;; or if explicitly requested
+            (if (or (symbolp (caar pairs)) (listp (caar pairs)))
+                (if (or (memq bbdb-update-address-class (car pairs))
+                        (and (assoc bbdb-update-address-class (car pairs))
+                             (string= bbdb-update-address-header
+                                      (cdr (assoc bbdb-update-address-class
+                                                  (car pairs))))))
+                    (setq pairs (cdr pairs))
+                  (setq pairs nil))
+              (if (not (and (eq 'authors bbdb-update-address-class)
+                            (string-match "From" bbdb-update-address-header)))
+                  (setq pairs nil)))
+
+            ;; now handle the remaining pairs
             (while pairs
               (setq regexp (car (car pairs))
                     string (cdr (car pairs)))
@@ -495,19 +514,20 @@ the variables `bbdb-auto-notes-alist' and `bbdb-auto-notes-ignore'."
                     (if replace-p
                         ;; replace old contents of field with STRING
                         (progn
-                          (if (eq notes-field-name 'notes)
-                              (message "Replacing with note \"%s\"" string)
+                          (when (not bbdb-silent-running)
+                            (if (eq notes-field-name 'notes)
+                                (message "Replacing with note \"%s\"" string)
                               (message "Replacing field \"%s\" with \"%s\""
-                                       notes-field-name string))
-                          (bbdb-record-putprop record notes-field-name
-                                               string)
+                                       notes-field-name string)))
+                          (bbdb-record-putprop record notes-field-name string)
                           (bbdb-maybe-update-display record))
                         ;; add STRING to old contents, don't replace
+                      (when (not bbdb-silent-running)
                         (if (eq notes-field-name 'notes)
                             (message "Adding note \"%s\"" string)
-                            (message "Adding \"%s\" to field \"%s\""
-                                     string notes-field-name))
-                        (bbdb-annotate-notes record string notes-field-name))))
+                          (message "Adding \"%s\" to field \"%s\""
+                                   string notes-field-name)))
+                      (bbdb-annotate-notes record string notes-field-name))))
               (setq pairs (cdr pairs))))
           (setq rest (cdr rest))))))
     (set-buffer b))))
@@ -661,4 +681,32 @@ For use as a value of `bbdb-change-hook'.  See `bbdb-net-redundant-p'."
        (bbdb-record-set-net record new)
        t))))
 
+
+
+;;;###autoload
+(defun bbdb-force-record-create ()
+  "Force automatic creation of a BBDB records for the current message.
+You might add this to the reply hook of your MUA in order to automatically
+get records added for those people you reply to."
+  (interactive)
+  (let ((bbdb/mail-auto-create-p t)
+        (bbdb/news-auto-create-p t)
+        (bbdb-message-caching-enabled nil)
+        (bbdb/gnus-update-records-mode 'annotating)
+        (bbdb/rmail-update-records-mode 'annotating)
+        (bbdb/mhe-update-records-mode 'annotating)
+        (bbdb/vm-update-records-mode 'annotating))
+    (save-excursion
+      (cond ((member major-mode '(vm-mode vm-virtual-mode vm-summary-mode
+                                          vm-presentation-mode))
+             (bbdb/vm-pop-up-bbdb-buffer))
+            ((member major-mode '(gnus-summary-mode gnus-article-mode
+                                                    gnus-tree-mode))
+             (bbdb/gnus-pop-up-bbdb-buffer))
+            ((member major-mode '(rmail-mode rmail-summary-mode))
+             (bbdb/rmail-pop-up-bbdb-buffer))
+            ((member major-mode '(mhe-mode mhe-summary-mode mh-folder-mode))
+             (bbdb/mh-pop-up-bbdb-buffer))
+            ))))
+
 (provide 'bbdb-hooks)
